@@ -1,6 +1,7 @@
 from gql import Client
 from gql.transport.requests import RequestsHTTPTransport
 from loguru import logger
+from typing import Union
 import base64
 
 import pandas as pd
@@ -9,16 +10,16 @@ import json
 import os
 import requests
 import getpass
+import warnings
 
 from .fileimport import FileImport
 from .automation import Automation, Schedule
 from .programming import Programming
+from .authorization import Authorization
 
 from .utils.ut import Utils, Structure, Defaults
 from .timeseries import TimeSeries
 
-import warnings
-
 class TechStack():
     """
     Initializes a Seven2one TechStack client.
@@ -54,6 +55,8 @@ class TechStack():
     copyGraphQLString: bool = False
         A developer feature: if True, with each method execution, 
         the graphQL string is copied to the clipboard (Windows only).
+    timeout: int = 30
+        Login timeout.
 
     Examples:
     >>> client = TechStack('app.orga.com/', 'admin', 'xxxx', logLevel='INFO')
@@ -72,13 +75,11 @@ class TechStack():
         dateTimeOffset: bool = True,
         raiseException: bool = False,
         copyGraphQLString: bool = False,
-        includeSystemProperties: bool = False,
         timeout: int = 30
         ) -> object:
-
-        warnings.filterwarnings('ignore')
-             
+            
         self.raiseException = raiseException
+        warnings.filterwarnings('ignore', message="The zone attribute is specific to pytz")
 
         try:
             logger.remove()
@@ -140,6 +141,8 @@ class TechStack():
         self.accessToken = token
         self.scheme = self.client.introspection
         self.proxies = proxies
+        self.host = host
+        self.user = user
 
         # Defaults:
         if timeZone == None:
@@ -148,7 +151,6 @@ class TechStack():
         self.defaults = Defaults(
             useDateTimeOffset=dateTimeOffset, 
             copyGraphQLString=copyGraphQLString,
-            includeSystemProperties=includeSystemProperties,
             timeZone=timeZone
             )
 
@@ -191,6 +193,7 @@ class TechStack():
             logger.debug(f"Reason: {err}")
 
         self.FileImport = FileImport(self)
+        self.Authorization = Authorization(self)
         return
 
     def _getAccessToken(self, tokenUrl: str, user: str, password: str, proxies: dict, timeout: int) -> str:
@@ -297,12 +300,15 @@ class TechStack():
         inventoryName:str, 
         references:bool=False, 
         fields:list=None,
-        where:str=None, 
-        orderBy:str=None, 
-        asc:bool=True, 
+        where:Union[list,tuple,str]=None, 
+        orderBy:Union[dict,list,str]=None, 
+        asc:Union[list,str]=True, 
         pageSize:int=5000, 
         arrayPageSize:int=50,
         top:int=100000,
+        validityDate:str=None,
+        allValidityPeriods:bool=False,
+        includeSysProperties:bool=False
         ) -> pd.DataFrame:
         """
         Returns items of an inventory in a DataFrame.
@@ -317,25 +323,44 @@ class TechStack():
         fields: list | str = None
             A list of all properties to be queried. If None, all properties will be queried.
             For referenced items use a '.' between inventory name and property.
-        where: str = None
-            Use a string to add where criteria like
-            'method eq "average" and location contains "Berlin"'.
-            Referenced items are not supported.
-        orderBy: str|list = None
-            Select one or more fields to sort by.
-        asc: bool|list = True
+        where: list | tuple | str = None
+            Define arguments critera like  'city eq "Berlin" and use lists for AND-combinations and
+            tuples for OR-combinations. For references use the format inventory.property as 
+            filter criteria.
+        orderBy: dict | list | str = None
+            Use a dict in the form of {property:'ASC'|'DESC'} or 
+            use a list of properties and the asc-argument for sorting direction
+        asc: list | bool = True
             Determines the sort order of items. If set to False, a descending order 
-            is applied.
+            is applied. Use a list, if more properties are selected in orderBy.
         pageSize: int = 5000
             The page size of items that is used to retrieve a large number of items.
         arrayPageSize: int = 50
             The page size of list items that is used to page list items in an inventory item.
         top: int = None
             Returns a restricted set of items oriented at the latest entries.
+        includeSysProperties: bool = False
+            If True, all system properties available will be queried. If False, 
+            only 'sys_inventoryItemId' will be queried by default. Despite of that, any system 
+            property can be passed to the fields argument as well.
+        validityDate: str = None
+            If the queried inventory has validity periods enabled, only items will be returned, 
+            which have the given timestamp between sys_validTo and sys_validFrom. Items without
+            validity periods are shown as well.
+        allValidityPeriods: bool = False
+            If True and if the queried inventory has validity periods enabled, all validity 
+            periods will be returned. If False, items with validity dates will not be returned.
+
 
         Example:
         --------
         >>> items('appartments', references=True)
+        >>> items(
+                'appartments',
+                fields=['address', 'owner'],
+                where=['city == "Berlin"', 'rooms > 2'],
+                orderBy={'size':'DESC', price:'ASC'}
+                )
         """
 
         if inventoryName not in self.inventory:
@@ -348,10 +373,14 @@ class TechStack():
         except:
             return
 
+        validityDate = Utils._argNone('validityDate', validityDate)
+        allValidityPeriods = Utils._argNone('allValidityPeriods', allValidityPeriods)
+        
+
         # core
         deleteId = False
         propertyDict = Utils._properties(self.scheme, inventoryName, recursive=True, 
-            sysProperties=self.defaults.includeSystemProperties)            
+            sysProperties=includeSysProperties)            
         if fields != None:
             if type(fields) != list:
                 fields = [fields]
@@ -370,6 +399,9 @@ class TechStack():
             Utils._error(self, f"Inventory '{inventoryName}' not found.")
             return
 
+        order = Utils._orderItems(self, orderBy, asc)
+        if order == None: return
+
         result = []
         count = 0
         stop = False
@@ -380,13 +412,16 @@ class TechStack():
             # Handling top (premature stop)
             if top != None:
                 loadedItems = pageSize * count
-                if top - loadedItems < pageSize:
+                if top - loadedItems <= pageSize:
                     stop = True
                     pageSize = top - loadedItems
 
             graphQLString= f''' query getItems {{
                     {inventoryName} (
-                            pageSize: {pageSize} 
+                            pageSize: {pageSize}
+                            {order}
+                            {allValidityPeriods}
+                            {validityDate}
                             {lastId}
                             {topLevelWhere}
                             ) {{
@@ -409,14 +444,7 @@ class TechStack():
             if stop == True: break
 
         df = pd.json_normalize(result)
-
-        if orderBy != None:
-            try:
-                df.sort_values(orderBy, ascending=asc, inplace=True, ignore_index=True)
-            except Exception as err:
-                Utils._error(self, err)          
-                return
-        
+       
         if fields != None: #sorts dataframe according to given fields
             try: # for array field this does not work
                 df = df[fields]
@@ -621,6 +649,66 @@ class TechStack():
             Utils._listGraphQlErrors(result, key)
         return result[key]['inventoryItems']
 
+    def addValidityItemsToParents(
+        self, 
+        inventoryName:str, 
+        items:list
+        ) -> str:
+        """
+        Adds from a list of dicts items with validity periods to an existing parent items. 
+        The 'sys_parentInventoryItemId' and either 'sys_validFrom' or 'sys_validFrom' are required
+        system properties.
+
+        Parameters:
+        -----------
+        inventoryName : str
+            The name of the inventory.
+        items : list
+            A list with dictionaries for each item.
+
+        Example:
+        --------
+        >>> items = [
+                {
+                'meterId': '86IEDD99',
+                'userId': 'DlvK5PCm4u',
+                'sys_parentInventoryItemId': 'EaM9zHA8Mi',
+                'sys_validFrom': '2023-12-31T23:00:00.000Z',
+                'sys_validTo': '2024-06-30T23:00:00.000Z',
+                },
+                {
+                'meterId': '86IEDD99',
+                'userId': 'DlvK5PCm8i',
+                'sys_parentInventoryItemId': 'EaM9zHA8Mi',
+                'sys_validFrom': '2024-07-31T23:00:00.000Z',
+                'sys_validTo': '2024-09-30T23:00:00.000Z',
+                }
+            ]
+        >>> addValidityItemsToParents('meterData', items)
+        """
+        items = Utils._propertiesToString(items)
+        key = f'create{inventoryName}ValidityPeriods'
+
+        graphQLString= f'''mutation addValidityPeriodItemsToParents {{
+            {key} (input: 
+                {items}
+            )
+                {{
+                    {Utils.errors}           
+                inventoryItems {{
+                    sys_inventoryItemId
+                    sys_versionId
+                }}
+            }}
+        }} 
+        '''
+        result = Utils._executeGraphQL(self, graphQLString)
+        if result == None: return
+
+        if result[key]['errors']:
+            Utils._listGraphQlErrors(result, key)
+        return result[key]['inventoryItems']
+
     def updateItems(
         self, 
         inventoryName:str, 
@@ -677,6 +765,56 @@ class TechStack():
         logger.info(f"{len(result[key]['inventoryItems'])} item(s) updated.")
         return
 
+    def updateDataFrameItems(
+        self,
+        inventoryName:str,
+        dataFrame:pd.DataFrame,
+        columns:list=None
+        ) -> None:
+
+        def convertDfToDict(df:pd.DataFrame, timeSeries:bool, columns:list=None) -> dict:
+
+            def createDict(df:pd.DataFrame) -> dict:
+                items = []
+                for _, row in df.iterrows():
+                    item = {}
+                    for header, value in zip(df.columns, row):
+                        if not pd.isna(value):
+                            item.setdefault(header, value)
+                    items.append(item)
+                return items
+
+            if timeSeries == True:
+                tsProperties = ['resolution', 'unit']
+                if columns == None:
+                    columns = [col for col in df.columns if col not in tsProperties]
+                else:
+                    columns = [col for col in columns if col not in tsProperties]
+
+            if columns == None:
+                return createDict(df)
+            else:
+                if 'sys_inventoryItemId' not in columns:
+                    columns.append('sys_inventoryItemId')
+                return df[columns].to_dict('records', )
+
+        if not 'sys_inventoryItemId' in dataFrame.columns:
+            logger.error("Missing column 'sys_inventoryItemId'. Items cannot be updated without this information.")
+            return
+
+        if self.structure[inventoryName]['variant'] == None:
+            items = convertDfToDict(dataFrame, False, columns=columns)
+        elif self.structure[inventoryName]['variant']['name'] not in ['TimeSeries', 'TimeSeriesGroup']:
+            items =  convertDfToDict(dataFrame, False, columns=columns)
+        else: 
+            items = convertDfToDict(dataFrame, True, columns=columns)
+
+        logger.debug(f"Items to write {items[:3]}...")
+
+        self.updateItems(inventoryName, items)
+        return
+
+
     def createInventory(
         self, 
         name:str,  
@@ -788,7 +926,10 @@ class TechStack():
         inventoryName:str, 
         force:bool=False
         ) -> None:
+
+        warnings.warn("'deleteInventory' is deprecated, use 'deleteInventories' instead.")
         """ 
+        THIS METHOD WILL DEPRECATE! Use 'deleteInventories()' instead.
         Deletes an inventory with all its containg items. 
         
         Parameters:
@@ -835,6 +976,71 @@ class TechStack():
                 logger.info(f"Inventory '{inventoryName}' deleted.")
         else: return
 
+    def deleteInventories(
+        self, 
+        inventoryNames:list,
+        deleteWithData:bool=False,
+        force:bool=False
+        ) -> None:
+        """ 
+        Deletes one or more inventories with the possibility to delete containing data.
+        
+        Parameters:
+        -----------
+        inventoryNames : list
+            A list of inventory names that should be deleted.
+        deleteWithData : bool = False
+            If True, containing data will be deleted.
+        force : bool
+            Use True to ignore confirmation.
+
+        Example:
+        ---------
+        >>> deleteInventories(['meterData'], deleteWitData=True, force=True)
+        """
+
+        inventoryIds = []
+        errorNames = []
+        for name in inventoryNames:
+            try:
+                inventoryIds.append(self.structure[name]['inventoryId'])
+            except:
+                errorNames.append(name)
+
+        if errorNames:
+            Utils._error(self, f"Unknown inventory names '{errorNames}'.")
+            return
+
+        _inventoryIds = Utils._graphQLList(inventoryIds)
+
+        if force == False:
+            confirm = input(f"Press 'y' to delete '{inventoryNames}': ")
+
+        key = 'deleteInventory'
+        graphQLString= f'''mutation deleteInventories {{
+            {key} (input:
+                {{
+                    inventoryIds: {_inventoryIds}
+                    ignoreData: {Utils._toGraphQL(deleteWithData)}
+                }}
+            )
+                {{
+                    {Utils.errors}           
+                }}
+            }}
+            '''
+
+        if force == True: confirm = 'y'
+        if confirm == 'y':
+            result = Utils._executeGraphQL(self, graphQLString)
+            if result == None: return
+            if result[key]['errors']:
+                Utils._listGraphQlErrors(result, key)
+                return
+            else: 
+                logger.info(f"Inventories '{inventoryNames}' deleted.")
+        else: return
+
     def variants(self) -> pd.DataFrame:
         """
             Returns a dataframe of available variants.
@@ -896,6 +1102,110 @@ class TechStack():
         else:
             logger.info(f"Variant deleted.")
 
+    def deleteItems(
+        self, 
+        inventoryName:str, 
+        inventoryItemIds:list=None, 
+        where:str=None, 
+        force:bool=False,
+        pageSize:int=500
+        ) -> None:
+        """
+        Deletes inventory items from a list of inventoryItemIds or by where-criteria. 
+
+        Parameters:
+        -----------
+        inventoryName: str
+            The name of the inventory.
+        inventoryItemIds: list = None
+            A list of inventoryItemIds that should be deleted.
+        where: str = None
+            Filter criteria to select items that should be deleted.
+        force: bool = False
+            Use True to ignore confirmation.
+        pageSize: int = 500
+            Only a limited amount of items can be deleted at once. 500 is default, however, 
+            if this size is too, big, choose a lower pageSize.
+
+        Examples:
+        ---------
+        >>> deleteItems('meterData', where='changeDate gt "2020-12-01"', force=True, pageSize=100)
+        >>> deleteItems('meterData', inventoryItemIds=['El5JrMG2xk'])
+        """
+
+        def delete(ids, n, m):
+
+            _ids = '['
+            for id in ids[n:m]: 
+                _ids += f'''{{sys_inventoryItemId: "{id}"}}\n'''
+            _ids += ']'
+
+            key = f'delete{inventoryName}'
+            graphQLString= f'''
+                mutation deleteItems {{
+                    {key} ( input: 
+                        {_ids}
+                    )
+                    {{
+                        {Utils.errors}           
+                    }}
+                }}
+                '''
+
+            result = Utils._executeGraphQL(self, graphQLString)
+
+            if result[key]['errors'] != None:
+                logger.error(Utils._listGraphQlErrors(result, key))
+                return
+
+        if inventoryItemIds == None and where == None:
+            Utils._error(self, f"No list of items and no where-criteria were provided.")
+            return
+                  
+        if inventoryItemIds != None and where != None:
+            logger.warning(f"List of items and where-criteria has been provided. Item list is used.")
+
+        if where != None:
+            _result = self.items(inventoryName, fields=['sys_inventoryItemId'], where=where)
+            if _result.empty:
+                logger.info(f"No results found for provided filter.")
+                return
+            ids = list(_result['sys_inventoryItemId'])
+        if inventoryItemIds != None:
+            _result = self.items(inventoryName, fields=['sys_inventoryItemId'], where=f'sys_inventoryItemId in {inventoryItemIds}')
+            if _result.empty:
+                logger.info(f"Provided id(s) could not be found.")
+                return
+            ids = list(_result['sys_inventoryItemId'])
+            diff = set(inventoryItemIds).difference(set(ids))
+            if diff:
+                Utils._error(self, f"The following item id's are not in the inventory: {ids}")
+                return
+                
+        logger.debug(f"GraphQL Ids: {ids}")
+
+        if force == False:
+            confirm = input(f"Press 'y' to delete  {len(ids)} items: ")       
+
+        if force == True: confirm = 'y'
+        if confirm == 'y':
+            n = 0
+            m = n + pageSize
+
+            while True:       
+                delete(ids, n, m)
+                n += pageSize
+                m += pageSize
+
+                if len(ids) - m < pageSize:
+                    delete(ids, n, len(ids))
+                    break
+        else:
+            return
+
+        logger.info(f"{len(ids)} items deleted.")
+        return
+
     def clearInventory(
         self, 
         inventoryName:str, 
@@ -975,86 +1285,6 @@ class TechStack():
         else:
             logger.info(f"{count} items deleted.")
 
-    def deleteItems(
-        self, 
-        inventoryName:str, 
-        inventoryItemIds:list=None, 
-        where:str=None, 
-        force=False
-        ) -> None:
-        """
-        Deletes inventory items from a list of inventoryItemIds or by where-criteria. 
-
-        Parameters:
-        -----------
-        inventoryName: str
-            The name of the inventory.
-        inventoryItemIds: list = None
-            A list of inventoryItemIds that should be deleted.
-        where: str = None
-            Filter criteria to select items that should be deleted.
-        force: bool = False
-            Use True to ignore confirmation.
-
-        Examples:
-        ---------
-        >>> deleteItems('meterData', where='changeDate gt "2020-12-01"', force=True)
-        >>> deleteItems('meterData', inventoryItemIds=['ef73d6d5-d1d7-459a-b079-ec640cbb310e'])
-        """
-
-        if inventoryItemIds == None and where == None:
-            Utils._error(self, f"No list of items and no where-criteria were provided.")
-            return
-            
-            
-        if inventoryItemIds != None and where != None:
-            logger.warning(f"List of items and where-criteria has been provided. Item list is used.")
-
-        if where != None:
-            _result =self.items(inventoryName, fields=['sys_inventoryItemId'], 
-                where=where)
-            if _result.empty:
-                logger.info(f"The where criteria '{where}' led to no results.")
-                return
-
-            _inventoryItemIds = list(_result['sys_inventoryItemId'])
-            
-        if inventoryItemIds != None:
-            _inventoryItemIds = inventoryItemIds
-
-        _ids = ''
-        for id in _inventoryItemIds:
-            _ids += f'{{sys_inventoryItemId: "{id}"}}\n'    
-      
-        logger.debug(f"GraphQL Ids: {_ids}")
-            
-        if force == False:
-            confirm = input(f"Press 'y' to delete  {len(_inventoryItemIds)} items: ")
-
-        key = f'delete{inventoryName}'
-        graphQLString= f'''
-            mutation deleteItems {{
-                {key} ( input: 
-                    [{_ids}]
-                )
-                {{
-                    {Utils.errors}           
-                }}
-            }}
-            '''
-
-        if force == True: confirm = 'y'
-        if confirm == 'y':
-            result = Utils._executeGraphQL(self, graphQLString)
-            if result == None: return
-        else:
-            return
-
-        if result[key]['errors'] != None:
-            Utils._listGraphQlErrors(result, key)
-        else:
-            logger.info(f"{len(_inventoryItemIds)} items deleted.")
-
     def updateVariant(
         self, 
         variantName, 
