@@ -87,8 +87,11 @@ class FileImport():
             pause:int = 1
             ) -> None:
         """
-        Imports  items from a CSV file. The CSV file only needs a header of
-        property definitions. Each line below the header represents a new item.
+        Imports items from a CSV file. The CSV file only needs a header of
+        property definitions. To connect an item to a reference item in another 
+        inventory, you can provide the reference property as header and the inventoryItemId 
+        or you choose a unique field from the reference inventory and use the format 
+        'inventory.property' in the header. Each line below the header represents a new item.
 
         Parameters:
         -----------
@@ -102,17 +105,14 @@ class FileImport():
             Determines the number of items which are written per chunk. Using chunks
             can be necessary to avoid overloading. Default is 500 items per chunk.
         pause : int = 1
-            Between each chunk upload a pause is inserted in order to avoid overloading.
+            Pause in seconds between each chunk upload to avoid overloading.
 
         Example:
         --------
-        >>> importItems(filePath='C:\\temp\\Items.csv', delimiter=';'
-            inventoryName='meterData')          
+        >>> importItems(filePath='C:\\temp\\Items.csv', inventoryName='meterData',
+                delimiter=';')          
         """
-        ## TIMEZONE
-        #timeZone = FileUtils._timeZone(timeZone)
-               
-        ## READ FILE
+              
         files = FileUtils._checkFilePath(filePath, self.raiseException)
         if files == None: return
         for file in files:
@@ -133,8 +133,9 @@ class FileImport():
             if len(referenceErrors) != 0:
                 Utils._error(f"Multilevel references such as {referenceErrors} are not supported.")
                 return
+            logger.debug(f"Reference fields: {referenceFields}")
 
-            dataType, isArray, nullable = FileUtils._analyzeProperties(inventoryName, properties)
+            dataType, isArray, nullable, isReference = FileUtils._analyzeProperties(inventoryName, properties)
             logger.info(f"File '{filePath}' read and properties analyzed")
 
             if len(referenceFields) != 0:
@@ -161,6 +162,118 @@ class FileImport():
 
         return
 
+    def importValidityItems(
+            self, 
+            filePath:str, 
+            inventoryName:str, 
+            delimiter:str=',',
+            uniquePropertyName:str=None,
+            chunkSize:int = 5000, 
+            pause:int = 1
+            ) -> None:
+        """
+        Imports items with validity periods from a CSV file. The CSV file needs a header of
+        property definitions. Specify a property that is unique to add new validity items to an
+        existing parent item, alternativly use the sys_inventoryItemId. If the unique property
+        is not existing yet, a new (parent) item will be created. Use the system properties 
+        'sys_validFrom' and 'sys_validTo' to specify validity periods. To connect an item to a 
+        reference item in another inventory, you can provide the reference property as header 
+        and the inventoryItemId or you choose a unique field from the reference inventory 
+        and use the format 'inventory.property' in the header. 
+        Each line below the header represents a new item.
+
+        Parameters:
+        -----------
+        filePath : str
+            The file path of the csv file that should be imported.
+        inventoryName : str
+            The field name of the inventory.
+        delimiter : str = ','
+            The CSV delimiter. Choose ',', ';', or 'tab'.
+        uniquePropertyName: str = None
+            If you want to add items with to an existing parent item, specify a property name that is 
+            unique ord use the 'sys_inventoryItemId'.
+        chunkSize : int = 5000
+            Determines the number of items which are written per chunk. Using chunks
+            can be necessary to avoid overloading. Default is 500 items per chunk.
+        pause : int = 1
+            Pause in seconds between each chunk upload to avoid overloading.
+
+        Example:
+        --------
+        >>> importValidityItems(filePath='C:\\temp\\Items.csv', inventoryName='meterData',
+        delimiter=';', uniquePropertyName='meterSerialNumber')          
+        """
+               
+        files = FileUtils._checkFilePath(filePath, self.raiseException)
+        if files == None: return
+        for file in files:
+            content = FileUtils._readCsvFile(file, delimiter)
+
+            ## PREPARE IMPORT   
+            properties = client.propertyList(inventoryName, references=True)
+            properties += ['sys_validFrom', 'sys_validTo']
+            logger.debug(f'Property names: {properties}')
+
+            diff = FileUtils._comparePropertiesBasic(properties, content[0])
+            if len(diff) > 0:
+                Utils._error(self, f"Unknown properties: {list(diff)}")
+                return 
+
+            properties = client.inventoryProperties(inventoryName)
+
+            referenceFields, referenceErrors = FileUtils._checkReferences(content[0])
+            if len(referenceErrors) != 0:
+                Utils._error(f"Multilevel references such as {referenceErrors} are not supported.")
+                return
+
+            dataType, isArray, nullable, isReference = FileUtils._analyzeProperties(inventoryName, properties)
+            logger.info(f"File '{filePath}' read and properties analyzed")
+
+            if len(referenceFields) != 0:
+                referenceMapping = FileUtils._createReferenceMapping(client, inventoryName, 
+                    referenceFields, isArray, content)
+            else:
+                referenceMapping = None
+
+            if uniquePropertyName != None:
+                parentMapping = FileUtils._createParentMapping(client, inventoryName, 
+                    content, uniquePropertyName)
+            else:
+                Utils._error(self, f"To import validity items, please specify a unique property")
+
+            validityItems, newItems = FileUtils._createItems(content, dataType, isArray, nullable, 
+                referenceMapping=referenceMapping, uniqueProperty=uniquePropertyName, parentItemMapping=parentMapping)
+            if len(newItems) == 0 and len(validityItems) == 0:
+                logger.info(f"The file '{file}' did not contain any valid items to import.")
+                continue
+            
+            # Import validity items
+            if len(validityItems) > 0:
+                if len(validityItems) > chunkSize:
+                    lenResult = 0
+                    for i in range(0, len(validityItems), chunkSize):
+                        result = client.addItems(inventoryName, validityItems[i : i + chunkSize])
+                        logger.info(f"{len(result) + lenResult} items of {len(validityItems)} imported.")
+                        sleep(pause)
+                else:
+                    result = client.addValidityItemsToParents(inventoryName, validityItems)
+                    logger.info(f"{len(result)} validity items of file '{filePath}' imported.")
+
+            # Import new items
+            if len(newItems) > 0:
+                if len(newItems) > chunkSize:
+                    lenResult = 0
+                    for i in range(0, len(newItems), chunkSize):
+                        result = client.addItems(inventoryName, newItems[i : i + chunkSize])
+                        logger.info(f"{len(result) + lenResult} items of {len(newItems)} imported.")
+                        sleep(pause)
+                else:
+                    result = client.addItems(inventoryName, newItems)
+                    logger.info(f"{len(result)} items of file '{filePath}' imported.")
+
+        return
+
     def importTimeSeriesItems(
         self, 
         filePath:str,  
@@ -185,14 +298,13 @@ class FileImport():
             Determines the number of items which are written per chunk. Using chunks
             can be necessary to avoid overloading. Default is 50 items per chunk.
         pause : int = 1
-            Between each chunk upload a pause is inserted in order to avoid overloading.
+            Pause in seconds between each chunk upload to avoid overloading.
                  
         """
 
         # if timeZone == None:
         #     timeZone = core._getDefaults()['timeZone']
                
-        ## READ FILE
         files = FileUtils._checkFilePath(filePath, self.raiseException)
         if files == None: return
         for file in files:
@@ -221,7 +333,7 @@ class FileImport():
                 Utils._error(f"Multilevel references such as {referenceErrors} are not supported.")
                 return
 
-            dataType, isArray, nullable = FileUtils._analyzeProperties(inventoryName, properties)
+            dataType, isArray, nullable, isReference = FileUtils._analyzeProperties(inventoryName, properties)
             logger.info(f"File '{filePath}' read and properties analyzed")
 
             if len(referenceFields) != 0:
@@ -336,7 +448,6 @@ class FileImport():
                 'Value errors': 0,
                 'Errors': None})
 
-            ## READ FILE
             content = FileUtils._readCsvFile(file, delimiter)
 
             tsLength = len(content) - 1
@@ -520,11 +631,10 @@ class FileImport():
             Determines the number of items which are written per chunk. Using chunks
             can be necessary to avoid overloading.
         pause: int = 1
-            Between each chunk upload a pause is inserted in order to avoid overloading.
+            Pause in seconds between each chunk upload to avoid overloading.
 
         """
               
-        ## READ FILE
         files = FileUtils._checkFilePath(filePath, self.raiseException)
         if files == None: return
 
@@ -562,7 +672,7 @@ class FileImport():
                     Utils._error(self, f"No item ids for importKeyProperty '{importKeyProperty}' found.")
                     return 
 
-            dataType, isArray, nullable = FileUtils._analyzeProperties(instanceInventoryName, properties)
+            dataType, isArray, nullable, isreference = FileUtils._analyzeProperties(instanceInventoryName, properties)
             logger.debug(f'Data types: {dataType}')
             logger.debug(f'Array properties: {isArray}')
             logger.debug(f'Nullable properties: {nullable}')
@@ -638,7 +748,7 @@ class FileImport():
             Determines the number of time series datapoints which are written per chunk. Using chunks
             can be necessary to avoid overloading.
         pause: int = 1
-            Between each chunk upload a pause is inserted in order to avoid overloading.
+            Pause in seconds between each chunk upload to avoid overloading.
         """
 
         ## TIMEZONE
@@ -661,7 +771,6 @@ class FileImport():
                 'Value errors': 0,
                 'Errors': None})
 
-            ## READ FILE
             content = FileUtils._readCsvFile(file, delimiter)
 
             ## PREPARE IMPORT: Get Instance properties
@@ -731,7 +840,7 @@ class FileImport():
                     return 
 
             ## PREPARE IMPORT: Check dataType, Array and nullable properties
-            dataType, isArray, nullable = FileUtils._analyzeProperties(instanceInventoryName, properties)
+            dataType, isArray, nullable, isReference = FileUtils._analyzeProperties(instanceInventoryName, properties)
             logger.info(f"File '{filePath}' read and properties analyzed")
 
             itemContent = FileUtils._createInstanceItemContent(content[:dataBegin])
@@ -1027,6 +1136,21 @@ class FileImport():
 
                 return output
 
+    def updateItems(
+        self,
+        filePath:str,
+        inventoryName,
+        delimiter:str=','
+        ) -> None:
+
+        files = FileUtils._checkFilePath(filePath, self.raiseException)
+        if files == None: return
+
+        df = pd.read_csv(filePath, sep=delimiter)
+
+        client.updateDataFrameItems(inventoryName, df)
+        return
+
     def templateImportReferences(
         self, 
         filePath:str=None
