@@ -10,13 +10,16 @@ from .utils.ut import Utils
 
 class Programming():
 
-    def __init__(self, accessToken:str, endpoint:str) -> None:
+    def __init__(self, accessToken:str, endpoint:str, client:object) -> None:
+
+        self.raiseException = client.raiseException
+        self.defaults = client.defaults
             
         header = {
             'authorization': 'Bearer ' + accessToken
         }
         
-        transport =  RequestsHTTPTransport(url=endpoint, headers=header, verify=True)
+        transport =  RequestsHTTPTransport(url=endpoint, headers=header, verify=True, proxies=client.proxies)
         self.client = Client(transport=transport, fetch_schema_from_transport=False)
 
         return
@@ -107,7 +110,12 @@ class Programming():
             return functionId
         else:
             if type(files) != list:
-                logger.error("Files must be of type list!")
+                msg = "Files must be of type list!"
+                if self.raiseException: raise Exception(msg)
+                else:
+                    logger.error(msg)
+                    return
+
             self.commitFunctionFiles(functionId, files)
             logger.info(f"Function with id {functionId} created.")
             if deploy == False:
@@ -179,7 +187,11 @@ class Programming():
                 return functionId
             else:
                 if deploymentName == None:
-                    logger.error(f"No deployment name was provided for deployment. Function id: {functionId}")
+                    msg = f"No deployment name was provided for deployment. Function id: {functionId}"
+                    if self.raiseException: raise Exception(msg)
+                    else:
+                        logger.error(msg)
+                        return
                 self.deployFunction(functionId, deploymentName, envVars, memoryLimit, cpuLimit)
 
         return result
@@ -278,7 +290,12 @@ class Programming():
 
         if downloadPath != None:
             if os.path.exists(downloadPath) == False:
-                logger.error(f"Download path '{downloadPath}' does not exist.")
+                msg = f"Download path '{downloadPath}' does not exist."
+                if self.raiseException: raise Exception(msg)
+                else:
+                    logger.error(msg)
+                    return
+
             else:
                 fileList = []
                 for file in result['functionFiles']['files']:
@@ -306,14 +323,13 @@ class Programming():
         if inputVariables == None:
             _vars = ''
         else:
-            _vars = json.dumps(inputVariables)
-            _vars = _vars.replace('"', '\\"')
+            _vars = Utils._toGraphQL(json.dumps(inputVariables))
 
         graphQLString = f''' mutation executeFunction {{
             executeFunction(
                 input: {{ 
-                    deploymentId: "{deploymentId}", 
-                    input: "{_vars}"
+                    deploymentId: {Utils._toGraphQL(deploymentId)},
+                    input: {_vars}
                     }}
             ) {{
                 executionId
