@@ -1,17 +1,14 @@
-from codecs import ignore_errors
 import pandas as pd
 import pytz
 from gql import Client
 from gql.transport.requests import RequestsHTTPTransport
 from loguru import logger
-from numpy import nan
 
 from seven2one.utils.ut_time import TimeUtils
 
 from .utils.ut import Utils
 from .utils.ut_timeseries import UtilsTimeSeries
 from time import time
-import sys
 
 class TimeSeries():
 
@@ -344,11 +341,11 @@ class TimeSeries():
         return
 
     def timeSeriesData(
-            self,
-            inventoryName:str,
-            fromTimepoint:str,
-            toTimepoint:str,
-            fields:list=['sys_displayValue'],
+            self, 
+            inventoryName:str, 
+            fromTimepoint:str, 
+            toTimepoint:str, 
+            fields:list=None, 
             where:str=None,
             unit:str=None,
             timeUnit:str=None,
@@ -372,11 +369,10 @@ class TimeSeries():
         toTimepoint: str
             The ending timepoint from where time series data will be retrieved
         fields: list|str = None
-            Properties of the time series to be used as header. Uses the displayValue as default.
-            If fields are not unique for each column, duplicates will be omitted. If you use
-            multiple fields, a MultiIndex DataFrame will be created. Use inventoryProperties()
-            to find out which properties are available for an inventory.
-            To access MultiIndex use syntax like <df[header1][header2]>.
+            Properties of the time series to be used as header. Uses the displayValue 
+            (inventoryItemId) as default. If fields are not unique for each column, 
+            duplicates will be omitted. If you use multiple fields, a MultiIndex DataFrame
+             will be created. To access MultiIndex use syntax like <df[header1][header2]>.
         where: str = None
             Use a string to add where criteria like
             'method eq "average" and location contains "Berlin"'
@@ -418,8 +414,8 @@ class TimeSeries():
         # fields
         deleteIdAfterwards = False
         if fields != None:
-            propertyDict = Utils._properties(self.scheme, inventoryName, recursive=True,
-                sysProperties=self.defaults.includeSystemProperties)
+            propertyDict = Utils._properties(self.scheme, inventoryName, recursive=True, 
+                sysProperties=False)
             if type(fields) != list:
                 fields = [fields]
             if 'sys_inventoryItemId' not in fields:
@@ -431,7 +427,12 @@ class TimeSeries():
                 Utils._error(self, f"Array type fields are not supported in 'fields'-argument ({_fields})")
                 return
         else:
-            fields = ['sys_displayValue', 'sys_inventoryItemId']
+            if self.structure[inventoryName]['displayValue'] != None:
+                fields = ['sys_displayValue', 'sys_inventoryItemId']
+                deleteIdAfterwards = True
+            else:
+                fields = ['sys_inventoryItemId']
+
             _fields = ''
             for field in fields:
                 _fields += field + '\n'
@@ -543,8 +544,8 @@ class TimeSeries():
             inventoryName:str, 
             fromTimepoint:str, 
             toTimepoint:str, 
-            fields:list=['sys_displayValue'], 
-            instanceFields:list=['sys_displayValue'], 
+            fields:list=None, 
+            instanceFields:list=None, 
             instancePrefix:str='instance.', 
             where:str=None, 
             whereInstance:str=None,
@@ -570,13 +571,13 @@ class TimeSeries():
         toTimepoint: str
             The ending timepoint from where time series data will be retrieved
         fields: list|str = None
-            Properties of the time series group to be used as header. Uses the displayValue as default. 
-            If fields are not unique for each column, duplicates will be omitted. If you use 
-            multiple fields, a MultiIndex DataFrame will be created. Use inventoryProperties() 
-            to find out which properties are available for an inventory. 
-            To access MultiIndex use syntax like <df[header1][header2]>.
+            Properties of the time series group to be used as header. Uses the displayValue 
+            (inventoryItemId) as default. If fields are not unique for each column, duplicates 
+            will be omitted. If you use multiple fields, a MultiIndex DataFrame will be 
+            created. To access MultiIndex use syntax like <df[header1][header2]>.
         instanceFields: list|str = None
-            Properties of the time series instance to be used as header. Uses the displayValue as default. 
+            Properties of the time series instance to be used as header. Uses the displayValue 
+            (inventoryItemId) as default. 
         instancePrefix: str = 'instance'
             Changes the prefix for all time series instance properties.
         where: str = None
@@ -597,7 +598,6 @@ class TimeSeries():
         timeZone: str = None
             A time zone provided in IANA or isoformat (e.g. 'Europe/Berlin' or 'CET'). Defaults
             to the local time zone or to a previously set default time zone.
-v
 
         Examples:
         ---------
@@ -630,29 +630,31 @@ v
                 fields += ['sys_inventoryItemId']
                 deleteIdAfterwards = True
         else:
-            fields = ['sys_displayValue', 'sys_inventoryItemId']
-            deleteIdAfterwards = True
+            if self.structure[inventoryName]['displayValue'] != None:
+                fields = ['sys_displayValue', 'sys_inventoryItemId']
+                deleteIdAfterwards = True
+            else:
+                fields = ['sys_inventoryItemId']
 
         _groupFields = ''
         for field in fields:
             _groupFields += field + '\n'
 
-        if type(instanceFields) != list:
-            instanceFields = [instanceFields]
+        if instanceFields != None:
+            if type(instanceFields) != list:
+                instanceFields = [instanceFields]
+        else:
+            instanceInventoryName = self.structure[inventoryName]['properties']['timeSeriesInstances']['inventoryName']
+            if self.structure[instanceInventoryName]['displayValue'] != None:
+                instanceFields  = ['sys_displayValue', 'sys_inventoryItemId']
+            else:
+                instanceFields  = ['sys_inventoryItemId']
 
         _instanceFields = ''
         for field in instanceFields:
             _instanceFields += field + '\n'
 
-        # resolvedFilter = ''
-        # if where != None: 
-        #     resolvedFilter = Utils._resolveWhereString(self, where)
         topLevelWhere, _ = Utils._handleWhere(self, where, inventoryName)
-
-        # resolvedInstanceFilter = ''
-        # if whereInstance != None: 
-        #     resolvedInstanceFilter = Utils._resolveWhereString(self, whereInstance)
-
         instanceWhere, _ = Utils._handleWhere(self, whereInstance, inventoryName)
 
         unit = Utils._argNone('unit', unit)
