@@ -19,7 +19,6 @@ class Defaults():
     timeZone: str = 'UTC'
     useDateTimeOffset: bool = True
     copyGraphQLString: bool = False
-    includeSystemProperties: bool = False
 
 class Structure():
     queryStructure = f'''query inventories {{
@@ -182,6 +181,7 @@ class Utils():
             return str(pytz.timezone(timeZone))
     
     def _graphQLList(itemList:list) -> str:
+        """Converts a list to a graphQL list"""
 
         def is_number(n):
             try:
@@ -189,8 +189,7 @@ class Utils():
             except ValueError:
                 return False
             return True
-
-        """Converts a list to a graphQL list"""
+        
         result = '['
         for item in itemList:
             if is_number(item):
@@ -257,7 +256,7 @@ class Utils():
             return ''
         return topLevelFilter, resolvedFilter
 
-    def _toGraphQL(value) -> Union[str, int, float]:
+    def _toGraphQL(value) -> Union[str, int, float, bool]:
         """
         Escapes a value for usage in GraphQL. Surrounding " will be added automatically.
 
@@ -327,8 +326,10 @@ class Utils():
                 'ne': 'neq',
                 'neq': 'neq',
                 'not in': 'not in',
-                'startswith': 'startswith',
-                'endswith': 'endswith',
+                'startsWith': 'startsWith',
+                'startswith': 'startsWith',
+                'endsWith': 'endsWith',
+                'endswith': 'endsWith',
                 '=': 'eq'
             }
             if operator in operators:
@@ -678,6 +679,49 @@ class Utils():
 
         logger.debug(f"returned property types: {propertyTypes}")
         return propertyTypes
+
+    def _orderItems(self, orderBy:Union[dict, list, str], asc:Union[list, str]=None) -> str:
+
+        def _orderDictToString(orderBy:dict) -> str:
+
+            order = 'order: ['
+            for key, value in orderBy.items():
+                if value not in ['ASC','DESC']:
+                    Utils._error(self, f"Invalid value '{value}' for property '{key}'. Use 'ASC' or 'DESC' instead.")
+                    return
+                order += f'{{{key}: {value}}}'
+            order += ']'
+            return order
+
+        mapping = {False:'DESC', True:'ASC', None:'ASC'}
+        if orderBy != None:
+            if type(orderBy) == dict:
+                order = _orderDictToString(orderBy)
+            if type(orderBy) == list:
+                if type(asc) == bool:
+                    _orderBy = {property:mapping[asc] for property in orderBy}
+                elif type(asc) == list:
+                    _asc = [mapping[item] for item in asc]
+                    _orderBy = dict(zip(orderBy, _asc))
+                elif asc == None:
+                    _orderBy = {property:'ASC' for property in orderBy}
+                else:
+                    Utils._error(self, f"Invalid type '{asc}' for order direction. Use bool or list of bools.")
+                order = _orderDictToString(_orderBy)
+            if type(orderBy) == str:
+                if type(asc) == bool:
+                    _orderBy = {orderBy:mapping[asc]}
+                elif type(asc) == list:
+                    _orderBy = {orderBy:mapping[asc[0]]}
+                elif asc == None:
+                    _orderBy = {orderBy:'ASC'}
+                else:
+                    Utils._error(self, f"Invalid type '{asc}' for order direction. Use bool instead.")
+                order = _orderDictToString(_orderBy)                  
+        else:
+            order = ''
+
+        return order
         
     def _copyGraphQLString(graphQLString:str, copyGraphQLString:bool=False) -> None:
         """Can be applied to any core function to get the GraphQL string which is stored in the clipboard"""
