@@ -106,9 +106,38 @@ class UtilsProperty():
 
 class FileUtils(UtilsProperty):
 
-    def _createItems(content:list, dataType:dict, isArray:list, nullable:list, referenceMapping=None, type=None):
+    def _createItems(
+        content:list,
+        dataType:dict, 
+        isArray:list, 
+        nullable:list, 
+        referenceMapping:dict=None, 
+        type:str=None,
+        uniqueProperty:str=None,
+        parentItemMapping:dict=None):
+        """
+        Creates the dictionary that is imported by 'addItems()'
+
+        Parameters:
+        ----------
+        content: list
+            The CSV content as list
+        dataType:dict
+            Property data types returned by UtilsProperty._getPropertyTypes()
+        isArray: list
+            Provides information, if the column is of array type. Returned by 
+            UtilsProperty._getArrays()
+        nullable: list
+            Provides information, if the column is nullable or not. Returned by
+            UtilsProperty._getNullables()
+        referenceMapping: dict = None
+            Provides mapping of unique properties with sys_inventoryItemIds. Returned by
+            FileUtils._createReferenceMapping()
+        type: str = None
+            If type is None, a 'basic' item is assumed. Another valid type is 'timeSeries'.
+        """
         
-        def _createItem():
+        def _createItem(header):
             if i == 0: return
             if len(row) < 1: return
             else:
@@ -116,20 +145,25 @@ class FileUtils(UtilsProperty):
                     item = {}
                 if type == 'timeSeries':
                     item = {'resolution': {}}
-                for j, field in enumerate(row):
-                    if not field:
+                for j, value in enumerate(row):
+
+                    if not value:
                         if header[j] in nullable:
                             continue
-                        elif header[j] not in nullable and len(field) > 0:
+                        elif header[j] not in nullable and len(value) > 0:
                             logger.warning(f"Line {i}, field '{header[j]}': Value missing for non nullable property.")
                             return
                         else:
                             pass
                     else:
+                        if parentItemMapping != None:
+                            if header[j] == uniqueProperty:
+                                if parentItemMapping[value] != None:
+                                    item.setdefault('sys_parentInventoryItemId', parentItemMapping[value])
                         if referenceMapping != None and header[j] in referenceMapping:
                             parentHeader = header[j].split('.')[0]
                             if parentHeader in isArray:
-                                nameList = UtilsProperty._transformList(field, 'String')
+                                nameList = UtilsProperty._transformList(value, 'String')
                                 try:
                                     idList = [referenceMapping[header[j]][item] for item in nameList]
                                 except KeyError as key:
@@ -138,49 +172,55 @@ class FileUtils(UtilsProperty):
                                 item.setdefault(parentHeader, idList)
                             else:
                                 try:
-                                    item.setdefault(parentHeader, referenceMapping[header[j]][field])
+                                    item.setdefault(parentHeader, referenceMapping[header[j]][value])
                                 except KeyError as key:
                                     logger.warning(f"Line {i}, field '{header[j]}': {key} is not a valid reference.")
                                     return
                         else:
+                            if header[j] == 'sys_inventoryItemId':
+                                if uniqueProperty == 'sys_inventoryItemId':
+                                    continue
                             if header[j] in dataType['Boolean']:
                                 if header[j] in isArray:
-                                    item.setdefault(header[j], UtilsProperty._transformList(field, 'Boolean'))
+                                    item.setdefault(header[j], UtilsProperty._transformList(value, 'Boolean'))
                                 else: 
-                                    item.setdefault(header[j], UtilsProperty._transformBool(field))
+                                    item.setdefault(header[j], UtilsProperty._transformBool(value))
                             elif header[j] in dataType['Numeric']:
                                 if header[j] in isArray:
-                                    item.setdefault(header[j], UtilsProperty._transformList(field,'Numeric'))
+                                    item.setdefault(header[j], UtilsProperty._transformList(value,'Numeric'))
                                 else:
-                                    item.setdefault(header[j], UtilsProperty._transformNumeric(field))
+                                    item.setdefault(header[j], UtilsProperty._transformNumeric(value))
                             elif header[j] in dataType['DateTime']:
                                 if header[j] in isArray:
-                                    item.setdefault(header[j], UtilsProperty._transformList(field, 'DateTime'))
+                                    item.setdefault(header[j], UtilsProperty._transformList(value, 'DateTime'))
                                 else:
-                                    #item.setdefault(header[j], FileUtils._convertTimestamp(field, timeZone, FileUtils._dateFormat(field)))
-                                    item.setdefault(header[j], field)
+                                    item.setdefault(header[j], value)
                             else:
                                 if type == 'timeSeries' and header[j] == 'timeUnit':
-                                    item['resolution'].setdefault('timeUnit', field)
+                                    item['resolution'].setdefault('timeUnit', value)
                                 elif type == 'timeSeries' and header[j] == 'factor':
-                                    item['resolution'].setdefault('factor', field)
+                                    item['resolution'].setdefault('factor', value)
                                 elif header[j] in isArray:
-                                    item.setdefault(header[j], UtilsProperty._transformList(field, 'String'))
+                                    item.setdefault(header[j], UtilsProperty._transformList(value, 'String'))
                                 else:
-                                    item.setdefault(header[j], field)
+                                    item.setdefault(header[j], value)
                 return item
                     
         itemList = []
+        validityItemList = []
         header = content[0]
         for i, row in enumerate(content):
-            item = _createItem()
-            if item != None:           
-                itemList.append(item)
-
-        return itemList          
-
+            item = _createItem(header)
+            if item != None:
+                if 'sys_parentInventoryItemId' in item:
+                    validityItemList.append(item)
+                else:
+                    itemList.append(item)
+        if all((uniqueProperty, parentItemMapping)) == False:
+            return itemList
+        else:
+            return (validityItemList, itemList)     
  
-
     def _createInstanceItems(content:list, dataType:dict, isArray:list, nullable:list, idMapping:dict=None, 
         transpose:bool=False) -> list:
 
@@ -328,7 +368,7 @@ class FileUtils(UtilsProperty):
         logger.debug(f'Nullable properties: {nullable}')
         logger.debug(f'Reference properties: {isReference}')
 
-        return dataType, isArray, nullable
+        return dataType, isArray, nullable, isReference
 
     def _timeZone(timeZone):
         if timeZone==None: 
@@ -477,7 +517,7 @@ class FileUtils(UtilsProperty):
 
         return references, errors
 
-    def _createReferenceMapping(client, inventoryName:str, referenceFields:list, isArray:list, content:list, ) -> dict:
+    def _createReferenceMapping(client:object, inventoryName:str, referenceFields:list, isArray:list, content:list) -> dict:
         """
         Creates a dictionary of mapping inventoryItemIds with the import items of reference.
         """
@@ -498,6 +538,8 @@ class FileUtils(UtilsProperty):
                 nameList = list(set().union(*listOfLists))
             else:
                 nameList = list(set(content[field]))
+
+            logger.debug(f"nameList: {nameList}")
             
             if sysId == False:
                 inventory = client.structure[inventoryName]['properties'][parentField]['inventoryName']
@@ -508,4 +550,60 @@ class FileUtils(UtilsProperty):
             
             references.setdefault(field, mapping)
 
-        return references
\ No newline at end of file
+        return references
+
+    def _createParentMapping(client:object, inventoryName:str, content:list, uniqueProperty:str ) -> dict:
+        """
+        Checks a list of fields unique fields whether they exist or not. Returns a dict of 
+        value:sys_inventoryItemId. Is used to import validity Items. a value is None, if a
+        sys_parentinventoryItemId does not exist.
+        """
+        content = pandas.DataFrame(content[1:], columns=content[0])
+        uniqueValues = list(content[uniqueProperty])
+        logger.debug(f"Unique value list: {uniqueValues[:4]}...")
+
+        if uniqueProperty == 'sys_inventoryItemId':
+            return {item:item for item in uniqueValues}
+
+        where = f'{uniqueProperty} in {uniqueValues}'
+        df = client.items(
+            inventoryName, 
+            fields=[uniqueProperty, 'sys_inventoryItemId'],
+            where=where
+            )
+
+        logger.debug(f"Dataframe for mapping: {df.head()}")
+
+        if not df.empty:
+            mapping = {item[1][uniqueProperty]: item[1]['sys_inventoryItemId'] for item in df.iterrows()}
+            remainingUniqueValues = list(set(uniqueValues).difference(set(df[uniqueProperty])))
+            logger.debug(f"Remaining unqique values: {remainingUniqueValues[:4]}...")
+            if len(remainingUniqueValues) > 0:
+                for item in remainingUniqueValues:
+                    mapping.setdefault(item, None)
+        else:
+            mapping = {item:None for item in uniqueValues}
+
+        logger.debug(f"Parent mapping: {mapping}")
+
+        return mapping
+            
+    def _readInventoryforMapping(
+            client:object,
+            inventoryName:str, 
+            property:str,
+            items:list) -> pandas.DataFrame:
+
+        filterList = '['
+        for item in items:
+            filterList += f'"{item}",' 
+        filterList += ']' 
+        
+        df = client.items(inventoryName, fields=[property,'sys_inventoryItemId'], 
+            where=f'{property} in {filterList}')
+        df = df.set_index(property)
+        logger.debug(f"Items to reference (first 5):\n {df.iloc[:5]}")
+        return df        
+        
+
+
