@@ -79,7 +79,7 @@ class UtilsTimeSeries():
             logger.info('The query did not produce results.')
             return df
         df.loc[(df.flag == 'MISSING'), 'value'] = nan
-
+        
         if displayMode == 'pivot':
             if includeMissing == False:
                 df = df.pivot_table(index='timestamp', columns=fields, values='value', dropna=True)
@@ -87,7 +87,7 @@ class UtilsTimeSeries():
                 df = df.pivot_table(index='timestamp', columns=fields, values='value', dropna=False)
 
             if len(df.columns) == 0:
-                logger.warning(f"No display value found. Provide at least 1 property for argument 'fields'.")
+                logger.warning(f"Could not pivot DataFrame. Try different properties for fields-argument or use displayMode='rows'")
                 return
             df.index = pd.to_datetime(df.index, format='%Y-%m-%dT%H:%M:%S').tz_convert(pytz.timezone(timeZone))
             if self.defaults.useDateTimeOffset == False:
